\\ Copyright 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(strictargs,1);
default(recover,0);
read("../vecper.gp");
read("../vecper-wip.gp");

\\ Usage: gp <continued-fraction.gp
\\
\\ In the usual way, a quadratic irrational is any solution to
\\
\\     a*x^2 + b*x + c = 0
\\
\\ for rationals a,b,c.  When discriminant b^2-4*a*c >= 0, the solutions are
\\ reals.  A real quadratic irrational has an eventually repeating continued
\\ fraction expansion.  For example
\\
\\     sqrt(41) = continued fraction 6, 2,2,12, 2,2,12, ... repeating
\\                        1
\\              = 6 + ------------
\\                             1
\\                    2  + ------------
\\                                  1
\\                         2  + ------------
\\                                      1
\\                              12 + ------------
\\                                             1
\\                                    2  + ------------
\\                                             ...
\\
\\ This is sqrt(41) because the repeating part
\\
\\                             1
\\              x =   2  + ------------
\\                                  1
\\                         2  + ------------
\\                                      1
\\                              12 + ------------
\\                                       x
\\
\\ is
\\              x = (62*x + 5) / (25*x + 2)
\\              25*x^2 - 60*x - 5 = 0
\\              
\\ which has solution
\\ 
\\              x = (60 + sqrt(4100)) / (2*25)
\\              x = (6 + sqrt(41))/5
\\
\\ then with the fixed initial part
\\
\\              6 + 1/x  =  sqrt(41)              
{
  contfracpnqn([2,2,12,x]) == [62*x + 5, 62;
                               25*x + 2, 25]   || error();
  ( x - (62*x + 5) / (25*x + 2) )*(25*x+2) == 25*x^2 - 60*x - 5  || error();
  poldisc(25*x^2 - 60*x - 5) == 4100           || error();
}

\\ Every repeating continued fraction is a quadratic in this way.
\\ vecper_contfrac() accepts a t_QUAD type real quadratic and gives its
\\ periodic continued fraction expansion.
\\
sqrt41 = quadgen(4*41);
sqrt41^2 == 41                               || error();
print("sqrt41 = contfrac ",vecper_contfrac(sqrt41));
vecper_contfrac(sqrt41) == [[6], [2,2,12]]   || error();

\\ The reverse operation, quad from a periodic vector, is
\\ vecper_contfrac_to_quad().
\\
vecper_contfrac_to_quad([[6], [2,2,12]]) == sqrt41   || error();

\\ It doesn't matter if the vecper has fixed and periodic part in minimum
\\ (normalized) form.  Anything with the same values is the same result.
\\
vecper_contfrac_to_quad([[6,2,2], [12,2,2, 12,2,2]]) == sqrt41   || error();


quit


\\-----------------------------------------------------------------------------
