\\ Copyright 2019, 2020, 2022, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose = 0;

read("vecper.gp");  \\ check multiple reads ok
read("vecper.gp");
read("vecper.gp");
read("test-helpers.gp");


\\-----------------------------------------------------------------------------
\\ vecper_INTERNAL_vector_rotate_after()

{
  check_equal(vecper_INTERNAL_vector_rotate_after([],0), []);
  check_equal(vecper_INTERNAL_vector_rotate_after([],-2), []);
  check_equal(vecper_INTERNAL_vector_rotate_after([99],-10), [99]);
}

{
  my(v=[10,20,30,40,50]);
  check_equal(vecper_INTERNAL_vector_rotate_after(v,0),
              [10,20,30,40,50]);
  check_equal(vecper_INTERNAL_vector_rotate_after(v,1),
              [20,30,40,50, 10]);
}

\\-----------------------------------------------------------------------------
\\ vecper_normalize()

check_equal(45/99, 5/11);
check_equal( 1/10*45/99, 1/22);

{
  check_equal(vecper_normalize([[0], [2,1,0]]), [[], [0,2,1]],
              "vecper_normalize() 0, 2,1,0");
  check_equal(vecper_normalize([[0], [1,0]]),   [[], [0,1]],
              "vecper_normalize() 0, 1,0");
  check_equal(vecper_normalize([[], [0,0,1]]), [[], [0,0,1]],
              "vecper_normalize() already");
  check_equal(vecper_normalize([[], [1,1]]), [[], [1]],
              "vecper_normalize() period halve");
  check_equal(vecper_normalize([[1,0,0], [1,1]]), [[1,0,0], [1]],
              "vecper_normalize() fixed unchanged, period halve");
  check_equal(vecper_normalize([[1,0,0], [0]]), [[1], [0]]);
  check_equal(vecper_normalize([[0,0,0,0,0,0,0,0,0], [0,0]]), [[], [0]]);

  my(elems=[[[[0,0], [0,1,3,6,2,5,0]],
             [[0], [0,0,1,3,6,2,5]] ],
            [[[10,20,30,40],[50,60,10,20,30,40]],
             [ [], [10,20,30,40,50,60] ] ],
            [[[99,20,30,40],[50,60,10,20,30,40]],
             [ [99], [20,30,40,50,60,10]] ],

            [[[99,20,30,40],[50,60,10,20,30,40, 
                             50,60,10,20,30,40,
                             50,60,10,20,30,40]],
             [ [99], [20,30,40,50,60,10]] ],

            [[ [99,88,30,40], [50,60,10,20,30,40]],
             [ [99,88], [30,40,50,60,10,20]] ]
           ]);
  for(i=1,#elems,
     my(v=elems[i][1],
        want=elems[i][2]);
     check_equal(vecper_normalize(v), want);
     check_equal(vecper_get_range(v,1,50),
                 vecper_get_range(want,1,50));
     my(flen=#want[1],
        plen=#want[2]);
     if(flen && plen,
        check_equal(want[1][flen] != want[2][plen], 1));
  );
}


\\-----------------------------------------------------------------------------
\\ vecper_make_state()

{
  check_equal(vecper_make_state((state)-> if(state<3, [10*state,state+1], 'end)),
              [[0,10,20],[]]);

  check_equal(
  vecper_make_state((state)->[10*state,(state+1)%3],
                    -1),
              [[-10], [0,10,20]],
              "vecper_make_state() addhelp example");
}


\\-----------------------------------------------------------------------------
\\ vecper_select()

check_equal(vecper_select(elem->elem%2==0, [[4,5],[6,7,8]]), \
            [[4],[6,8]], "vecper_select()");
check_equal(vecper_select(elem->0, [[4,5],[6,7,8]]), \
            [[],[]]);

\\-----------------------------------------------------------------------------
\\ vecper_get_range()

check_equal(vecper_get_range([[4,5],[6,7,8]], 1,2), [4,5], \
            "vecper_get_range()");
check_equal(vecper_get_range([[4,5],[6,7,8]], 2,4), [5,6,7]);

test_get_range(v,lo,hi) = 
{
  \\ print("vecper_get_range() "v" lo="lo" hi="hi);
  if(hi==lo-1, [],     \\ GP 2.7.x allow crossed range for nothing
     hi <= #v[1] || #v[2]==0,  v[1][lo..hi],
     my(ret=vector(ceil((hi-lo)/#v[2])+2, i, []),
        from=1,
        upto=0);
     \\ print("ret length "#ret);
     my(len=hi-lo+1);
     if(lo > #v[1],
        from=2;
        lo = (lo-#v[1]-1)%#v[2]+1;
        \\ print("  wrap to lo="lo);
       );
     while(len>0,
           \\ print("  from "from" len="len" lo="lo);
           my(t=min(len,#v[from]-lo+1));
           ret[upto++] = v[from][lo..lo+t-1];
           len-=t;
           from=2; lo=1);
     \\ print("  final "ret);
     concat(ret));           
}

{
  for(fixed=0,4,
     my(v_fixed=vector(fixed,i, 10*i));
     for(periodic=0,4,
        my(v_periodic=vector(periodic,i, 100*i),
           v = [v_fixed,v_periodic],
           end=if(periodic==0,fixed, fixed+4*periodic));
        for(lo=1,end,
           for(hi=lo-1,end,
              \\ print("vecper "v"  lo="lo" hi="hi);
              check_equal(vecper_get_range(v,lo,hi),
                          test_get_range(v,lo,hi))))));
}

\\-----------------------------------------------------------------------------
\\ cmp()

check_equal(cmp([1],[2]), -1);
check_equal(cmp([1,9],[2]), 1);  \\ length first, then values
check_equal(cmp([0,0],[9]), 1);  \\ length first, then values


\\-----------------------------------------------------------------------------
\\ vecper_lex()

check_equal(cmp(simplify([ 1+I-I, [2+I-I]]), [1,[2]]), 0, "cmp of simplify");
check_equal(cmp(         [ 1+I-I, [2+I-I]] , [1,[2]]), 0, "cmp");
{
  my(z=quadgen(12)); z+=1; z-=quadgen(12);
  check_equal(type(z), "t_QUAD");
  check_equal(z==1, 1);
  if(have_lex_of_quad,
     check_equal(lex(z,1), 0,
                 "lex() 1 == 1+0*I in quad"));
}
\\ check_equal(lex("abc", 1), 0);    \\ no mixed types
\\ check_equal(lex(1, Pol(1)), 0);   \\ no mixed types

check_equal(vecper_lex([[0],[1,0]], [[0],[1,0]]), 0, \
            "vecper_lex() identical");
check_equal(vecper_lex([[1], []], [[2], []]), -1, \
            "vecper_lex() finite smaller");
check_equal(vecper_lex([[1,99], []], [[2], []]), -1, \
            "vecper_lex() finite smaller");
check_equal(vecper_lex([[1,99], [10]], [[2], []]), -1, \
            "vecper_lex() smaller");
check_equal(vecper_lex([[1,0,1,0,1,0,1,0], [1,2]], [[], [1,0]]), 1, "lex bigger");
check_equal(vecper_lex([[], [1,0,1,0,1,2]], [[], [1,0]]), 1, "lex bigger");
check_equal(vecper_lex([[0,0,0,0],[]], [[],[]]), 1);
check_equal(vecper_lex([[],[]], [[0,0,0,0],[]]), -1);
check_equal(vecper_lex([[0,0,0,0],[]], [[], [1]]), -1, "smaller");
check_equal(vecper_lex([[0,0,0,0],[1]], [[],[]]), 1);
check_equal(vecper_lex([[0,0,0,0],[1]], [[],[0,0,0,0,2]]), -1);

check_equal(lex(1, 1.0), 0, "lex() 1 == 1.0");
check_equal(lex([1], [1.0]), 0, "lex() [1] == [1.0]");
check_equal(vecper_lex([[1],[]], [[1.0],[]]), 0, \
            "vecper_lex() 1 == 1.0");

check_equal(lex([1,1,1,1], [1]), 1);
check_equal(vecper_lex([[],[1]], [[1],[]]), 1, \
            "vecper_lex() infinite > finite");

check_equal(lex([1], [1,1,1,1]), -1);
check_equal(vecper_lex([[1],[]], [[],[1]]), -1, \
            "vecper_lex() finite < infinite");

{
  for(len1=0,4,
     for(pos1=1,len1,
        for(s1=-1,1,
           my(vec1=vector(len1));
           vec1[pos1]=s1;

           for(len2=0,4,
              for(pos2=1,len2,
                 for(s2=-1,1,
                    my(vec2=vector(len2));
                    vec2[pos2]=s2;

                    check_equal(lex(vec1,vec2),
                                vecper_lex([vec1,[]], [vec2,[]]))))))));
}


\\-----------------------------------------------------------------------------
\\ vecper_is_equal()

check_equal(lcm(0,0), 0);
check_equal(lcm(2,2), 2);

{
  check_equal(vecper_is_equal([[1],[]], [[1],[]]), 1,
              "vecper_is_equal() identical");
  check_equal(vecper_is_equal([[1],[0,0]], [[1],[0]]), 1,
              "vecper_is_equal() though different periods");

  check_equal(vecper_is_equal([[1],[]], [[1,2],[]]), 0,
              "vecper_is_equal() different length finite");
  check_equal(vecper_is_equal([[1,2],[]], [[1,5],[]]), 0,
              "vecper_is_equal() same length finite");
  check_equal(vecper_is_equal([[1,2],[]], [[1,2],[]]), 1,
              "vecper_is_equal() same length finite");
}

check_equal(1==1.0, 1, "1==1.0");
check_equal(cmp(1, 1.0), -1, "cmp() 1 < 1.0");
check_equal(vecper_is_equal([[1],[]], [[1.0],[]]), 1, "vecper_is_equal() 1 == 1.0");


\\-----------------------------------------------------------------------------
\\ vecper_fixed()
\\ vecper_periodic()
\\ vecper_fixed_length()
\\ vecper_periodic_length()

check_equal(vecper_fixed([[4,5],[6,7,8]]), [4,5])
check_equal(vecper_periodic([[4,5],[6,7,8]]), [6,7,8])

check_equal(vecper_fixed_length([[4,5],[6,7,8]]), 2)
check_equal(vecper_periodic_length([[4,5],[6,7,8]]), 3)


\\-----------------------------------------------------------------------------
\\ vecper_apply()

check_equal(vecper_apply(d->10*d, [[4,5],[6,7,8]]), \
            [[40,50],[60,70,80]], "vecper_apply()");


\\-----------------------------------------------------------------------------
\\ vecper_apply_state()

{
  \\ example in the addhelp()
  check_equal(
  vecper_apply_state((elem,state)-> elem*=state; state*=-1; [elem,state],
                     [[4,5],[6,7,8]],
                     1)
,
              [[4,-5], [6,-7,8,-6,7,-8]],
              "vecper_apply_state()");
}
{
  my(v=[[1,2], [3,4,5]]);
  check_equal(vecper_apply_state((elem,state)->[elem,state],v),
              v,
              "vecper_apply_state() noop");
}
{
  my(v=[[], [0,1]]);
  check_equal(vecper_apply_state((elem,state)->
                                 elem*=state;
                                 if(elem,state*=-1);
                                 [elem,state],
                                 v,
                                 1),
              [[], [0,1,0,-1]],
              "vecper_apply_state() flip");
}

{
  my(v=[[4],[6,7,8]]);
  check_equal(vecper_apply_state((elem,state)->
                                 elem*=10^state;
                                 state++; if(state>3,state=1);
                                 [elem,state],
                                 v,
                                 -2),
              [[4/100, 6/10, 7, 80], [600, 7000, 80]]);
}

test_negate_below_01(bit,state) =
{
  \\ print("  bit="bit" state "state);
  my(s=state[1],
     prev=state[2],
     new_bit = bit*s);          \\ possible negate
  if(prev==0 && bit==1, s=-s);  \\ negate below each 01
  state=[s,bit];                \\ new state
  \\ print("    return "new_bit" state "state);
  [new_bit,state];
}
{
  check_equal(vecper_apply_state(test_negate_below_01, [[], [0]], [1,0]),
              [[], [0]]);
  check_equal(vecper_apply_state(test_negate_below_01, [[], [1,1]], [1,0]),
              [[1,-1], [-1,-1]]);
  check_equal(vecper_apply_state(test_negate_below_01, [[], [1]], [1,0]),
              [[1], [-1]]);
}

\\-----------------------------------------------------------------------------
\\ vecper_polrev()

check_equal(Polrev([10,11,12]), 10+11*x+12*x^2);
check_equal(vecper_polrev([[],[]],'y), 0*'y);
check_equal(vecper_polrev([[10,11,12], []]), 10+11*x+12*x^2);

{
  local(x=123);
  check_equal(vecper_polrev([[0,1],[]]), 'x);
}

{
  check_equal(y,'y);
  check_equal(vecper_polrev([[10,11,12], [7,8,9]], y) + O(y^10),
              10 + 11*y + 12*y^2 
              + 7*y^3 + 8*y^4 + 9*y^5 
              + 7*y^6 + 8*y^7 + 9*y^8 
              + 7*y^9 + O(y^10));
}
check_equal(vecper_polrev([[4],[5,6]]), \
            4 + x*(5 + 6*x)/(1-x^2));
check_equal(vecper_polrev([[4],[5,6]]) + O(x^5), \
            4 + 5*x + 6*x^2 + 5*x^3 + 6*x^4 + O(x^5));

\\-----------------------------------------------------------------------------
\\ vecper_get()

{
  my(v=[[10,11], [12,13,14]]);
  check_equal(vector(10,i, vecper_get(v,i)),
              [10,11, 12,13,14, 12,13,14, 12,13],
              "vecper_get()");
}

\\-----------------------------------------------------------------------------
\\ vecper_INTERNAL_vector_periodic_reduce()

check_equal(vecper_INTERNAL_vector_periodic_reduce([]), [], \
           "vecper_INTERNAL_vector_periodic_reduce() empty");
check_equal(vecper_INTERNAL_vector_periodic_reduce([0,0,0]), [0], \
           "vecper_INTERNAL_vector_periodic_reduce() three");
check_equal(vecper_INTERNAL_vector_periodic_reduce([1,2,1,2]), [1,2], \
            "vecper_INTERNAL_vector_periodic_reduce()");
check_equal(vecper_INTERNAL_vector_periodic_reduce([1,2,1,2,3]), [1,2,1,2,3], \
            "vecper_INTERNAL_vector_periodic_reduce()");

\\-----------------------------------------------------------------------------
\\ vecper_frac_fromdigits()

check_equal(Pol([0,1,0]),'x, "Pol 0,1,0");
check_equal(Pol([1,0]),  'x);
check_equal(Pol([0,1])==1, 1, "Pol");

{
  check_equal(vecper_frac_fromdigits([[], [1,0,0,0]],2),
              8/15, ".10001000 binary");

  \\ [+1]000[-1]000...
  check_equal(vecper_frac_fromdigits([[], [0,1,1,1, 1,0,0,0]],2),
              8/17, ".01111000 binary");
  check_equal((128 - 8)/(256-1), 8/17);
}

check_equal(vecper_frac_fromdigits([[], [1, 4, 2, 8, 5, 7]]), 1/7, "1/7 decimal");
check_equal(vecper_frac_fromdigits([[], [0,9]]), 1/11, "1/11 decimal");
check_equal(vecper_frac_fromdigits([[0], [4,5]]), 1/22, "1/11 decimal");
check_equal(vecper_frac_fromdigits([[], [0,0,1]],2), 1/7, "0,0,1 repeat");
check_equal(vecper_frac_fromdigits([[], [0,1]],2), 1/3, "0,1 repeat");
check_equal(vecper_frac_fromdigits([[], [1,0]],2), 2/3, "1,0 repeat");
check_equal(vecper_frac_fromdigits([[0], [1,0]],2), 1/3, "1,0 repeat, half");

check_equal(vecper_frac_fromdigits([[], [9]]), 1, "9 repeat");
check_equal(vecper_frac_fromdigits([[], [9,9]]), 1, "9 repeat, two");
check_equal(vecper_frac_fromdigits([[], [9,9,9]]), 1, "9 repeat, three");
check_equal(vecper_frac_fromdigits([[], [1]],2), 1, "1 repeat");
check_equal(vecper_frac_fromdigits([[], [1,1]],2), 1, "1 repeat, two");
check_equal(vecper_frac_fromdigits([[], [1,1,1]],2), 1, "1 repeat, three");
check_equal(vecper_frac_fromdigits([[1], []],2), 1/2, "1 only = 1/2");
check_equal(vecper_frac_fromdigits([[0], [1]],2), 1/2, "1 repeat = 1/2");
check_equal(vecper_frac_fromdigits([[0], [1,1]],2), 1/2, "1 repeat two, half");
check_equal(vecper_frac_fromdigits([[1], [1,1]],2), 1, "[1], [1,1]");
check_equal(vecper_frac_fromdigits([[], [0]],2), 0, "[], [0]");
check_equal(vecper_frac_fromdigits([[0], [0]],2), 0, "[0], [0]");

\\ x^2*1/10 + x^2*1/100 + ... = x^2*1/9
check_equal(vecper_frac_fromdigits([[], [x^2]]), 1/9*x^2, \
            "polynomial elements of v");

\\ x^2*1/y + x^2*1/y^2 + ... = x^2*1/(y-1)
check_equal(vecper_frac_fromdigits([[], [x^2]], y), x^2*1/(y-1), \
            "polynomial elements of v");


\\-----------------------------------------------------------------------------
\\ vecper_add()

\\ check_equal(vecper_add([1,2,3,[4,5]],[100,200,300,[400,500]]), \
\\             [101,202,303,[404,505]]);
\\ check_equal(vecper_add([1,2,3,[]],[100,200,[]]), \
\\             [101,202,3,[]]);
\\ 
\\ check_equal(vecper_frac_digits(1/7), [[0, 0, 1]]);
\\ check_equal(vecper_frac_digits(1/13),[[0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1]]);
\\ check_equal(vecper_add(vecper_frac_digits(1/7),vecper_frac_digits(1/13)), \
\\             vecper_frac_digits(1/7+1/13));
\\ quit

\\ test_poly_terms(p,n) =
\\ {
\\   my(v=vector(n));
\\   for(i=1,#v,
\\      p*=variable(p);
\\      v[i]=numerator(p)\denominator(p);
\\      p-=v[i]);
\\   v;
\\ }
\\ check_equal(test_poly_terms(1/x + 2/x^2, 4), [1,2,0,0]);
\\ check_equal(test_poly_terms(1/x + 2/x^3, 4), [1,0,2,0]);
\\ 
\\ {
\\   my(want_poly = 1/x + 1/x*3/(x^2-1));
\\   check_equal(test_poly_terms(want_poly,7), [1,0,3,0,3,0,3], "poly terms");
\\   check_equal(vecper_frac_fromdigits([1,[0,3]],'x), want_poly, "addhelp example");
\\ }


\\-----------------------------------------------------------------------------
\\ vecper_frac_digits()

check_equal(vecper_frac_digits(0.5), [[5], [0]], \
            "vecper_frac_digits() 0.5");
check_equal(vecper_frac_digits(0.25), [[2,5], [0]], \
            "vecper_frac_digits() 0.25");
check_equal(vecper_frac_digits(1.0), [[], [9]], \
            "vecper_frac_digits() 1.0");
check_equal(vecper_frac_digits(0.0), [[], [0]], \
            "vecper_frac_digits() 0.0");

check_equal(vecper_frac_digits(1/10), [[1], [0]], \
            "vecper_frac_digits() 1/10");
check_equal(vecper_frac_digits(1/10,,1), [[0], [9]], \
            "vecper_frac_digits() 1/10 nines");
check_equal(vecper_frac_fromdigits([[0], [9]]), 1/10);
check_equal(vecper_frac_fromdigits([[1], [0]]), 1/10);

check_equal(vecper_frac_digits(0), [[], [0]], \
            "vecper_frac_digits() 0");
check_equal(vecper_frac_digits(0,,1), [[], [0]], \
            "vecper_frac_digits() 0 nines, is same");

check_equal(vecper_frac_digits(1), [[], [9]], \
            "vecper_frac_digits() 1");
check_equal(vecper_frac_digits(1,,1), [[], [9]], \
            "vecper_frac_digits() 1 nines, is same");

check_equal(vecper_frac_digits(6/7), [[], [8,5,7,1,4,2]], \
            "vecper_frac_digits() 6/7 base 10");
check_equal(vecper_frac_digits(1/8, 4), [[0,2], [0]], \
            "vecper_frac_digits() 1/8 base 4");

check_equal(vecper_frac_digits(1/11,10), [[], [0,9]], \
            "vecper_frac_digits() 1/11");
check_equal(vecper_frac_digits(1/22,10), [[0], [4,5]], \
            "vecper_frac_digits()() 1/22");
check_equal(vecper_frac_digits(1/7,10), [[], [1, 4, 2, 8, 5, 7]]);
check_equal(vecper_frac_digits(1/2,2), [[1], [0]]);
check_equal(vecper_frac_digits(1/4,2), [[0,1], [0]]);
check_equal(vecper_frac_digits(1/4,2), [[0,1], [0]]);
check_equal(vecper_frac_digits(1/3,2), [[], [0,1]], "vecper_frac_digits() 1/3");
check_equal(vecper_frac_digits(2/3,2), [[], [1,0]], "2/3");
check_equal(vecper_frac_digits(1/7,2), [[], [0,0,1]], "1/7");
check_equal(vecper_frac_digits(3/7,2), [[], [0,1,1]], "3/7");
check_equal(vecper_frac_digits(1/14,2), [[0], [0,0,1]], "1/14");
check_equal(vecper_frac_digits(16/17,2), [[], [1, 1, 1, 1, 0, 0, 0, 0]], "8/9");
check_equal(vecper_frac_digits(1023/1024,2), [[1,1,1,1,1,1,1,1,1,1], [0]]);
check_equal(vecper_frac_digits(1,2), [[], [1]], "1/1 binary");
check_equal(vecper_frac_digits(1/13,2), [[], [0,0,0,1, 0,0,1, 1,1,0, 1,1]]);

{
  for(base=2,12,
     \\ print("base=",base);
     for(den=1,30,
        \\ print("den=",den);
        for(num=0,den,
           my(f=num/den);
           \\ print("try f="f);
           my(v=vecper_frac_digits(f,base));
           \\ print("  v="v);
           my(got=vecper_frac_fromdigits(v,base));
           check_equal(got,f))));
}

{
  my(v=vecper_frac_digits(1/277));
   check_equal(fromdigits(vecper_periodic(v)),
              3610108303249097472924187725631768953068592057761732851985559566787,
              "vecper_frac_digits() 1/277");
}


\\==============================================================================
\\ No globals left over.
\\ test-symbols.txt is all symbols appearing in vecper.gp.

check_equal(vecper,'vecper);
{
  my(test_symbols=readstr("test-symbols.txt"));
  check_equal(#test_symbols>0,1);
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     check_equal(eval(test_symbol),
                 eval(Str("'"test_symbol)),
                 Str("leftover global variable: "test_symbol)));
}
\\ {
\\   my(test_symbols=readstr("test-funcs.txt"));
\\   for(test_i=1,#test_symbols,
\\      my(test_symbol=test_symbols[test_i]);
\\      my(value=eval(test_symbol));
\\      check_equal(value == eval(Str("'"test_symbol))
\\                  || type(value) == "t_CLOSURE", 1,
\\                  Str("leftover non-function: "test_symbol)));
\\ }

\\-----------------------------------------------------------------------------

print("total "check_count" tests, time ",gettime()\1000," seconds");
